/*
** Module   : XEntityImpl
** Abstract : DOM XML document and nodereference commons implementation.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 EVL 20130130 Created initial version.
*/

package com.goldencode.p2j.xml;

import java.util.*;
import org.w3c.dom.*;
import com.goldencode.p2j.util.*;

/**
 * Implementation of the common features for X-Docunet and X-Node-Reference
 * Progress DOM XML support objects.
 */
public class XEntityImpl
implements XEntity
{
   /** The handle object associated with this node or document. */
   private handle hXEntity = null;

   /** List of child node names. */
   private Map<handle, character> childNodeNameList;

   /** List of child node names. */
   private Map<handle, character> childNodeTypeList;

   /**
    * Default constructor.
    */
   public XEntityImpl()
   {
      hXEntity = new handle();
      
      childNodeNameList  = Collections.synchronizedMap(
                                     new LinkedHashMap<handle, character>());
      childNodeTypeList  = Collections.synchronizedMap(
                                     new LinkedHashMap<handle, character>());
   }

   /**
    * Special constructor accepting the X-Document or X-Node-Reference object.
    * 
    * @param    value
    *           The object to be associated with the given handle object.
    */
   public XEntityImpl(WrappedResource value)
   {
      this();
      hXEntity.assign(value);
   }

   /**
    * Getting the handle of the object.  
    *
    * @return   Internal handle obect associated with this node.
    */
   public handle getHandle()
   {
      return hXEntity;
   }
   
   /**
    * Reports if this object is valid for use.  
    *
    * @return   <code>true</code> if we are valid (can be used).
    */
   public boolean isValid()
   {
      return hXEntity != null;
   }
   
   /**
    * Reports if this object is unknown.  
    *
    * @return   <code>true</code> if object is unknown.
    */
   public boolean isUnknown()
   {
      return hXEntity == null;
   }

   // Attributes
   /**
    * Gets the the number of children objects for the current one.
    * 
    * @return   The integer number of the children.
    */
   public integer getNumChildren()
   {
      return new integer(childNodeNameList.size());
   }

   /**
    * Gets the the unique ID number associated to this object by the
    * underlying system. Not the same as the handle value itself.
    * 
    * @return   The integer number of the children.
    */
   public integer getUniqId()
   {
      return new integer(hXEntity.hashCode());
   }

   // Methods
   /**
    * Appending the new child to the current object.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be appended.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical appendChild(handle hChild)
   {
      return appendChild(hChild, (character) null, (character) null);
   }

   /**
    * Appending the new child to the current object.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be appended.
    * @param    name
    *           The name of the child to be appended.
    * @param    type
    *           The type of the child to be appended.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical appendChild(handle hChild, character name, character type)
   {
      logical res = new logical(true);
      
      childNodeNameList.put(hChild, name);
      childNodeTypeList.put(hChild, type);
      
      return res;
   }

   // Returning value placed into hChild
   /**
    * Getting the handle object of the child node with specified index.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to put
    *           the results.
    * @param    index
    *           1-based index of the child node to be retrieved.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical getChild(handle hChild, int index)
   {
      return getChild(hChild, new integer(index));
   }
   
   /**
    * Getting the handle object of the child node with specified index.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to put
    *           the results.
    * @param    index
    *           1-based index of the child node to be retrieved.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical getChild(handle hChild, integer index)
   {
      int ndx = index.getValue();
      
      if(ndx < 1 || ndx > childNodeNameList.size())
         return new logical(false);
      
      Set keySet = childNodeNameList.keySet();
      Iterator iter = keySet.iterator();
      // Skip keys before one we look at.
      for(; ndx > 1; ndx--)
      {
         iter.next();
      }
      // And get the required key
      if(iter.hasNext())
         hChild = (handle)iter.next();
      else
         return new logical(true);
      
      return new logical(true);
   }
   
   /**
    * Removing the specified child from the node.
    * 
    * @param    hChild
    *           The valid handle class instance of the child to be removed.
    * 
    * @return   The <code>true</code> in case of success
    *           <code>false</code> otherwise.
    */
   public logical removeChild(handle hChild)
   {
      return new logical(childNodeNameList.remove(hChild) != null &&
                         childNodeTypeList.remove(hChild) != null);
   }
   
   /**
    * Clean up the current node.
    */
   public void clear()
   {
      childNodeNameList.clear();
      childNodeTypeList.clear();
      hXEntity = null;
   }
}
